@@ -1,22 +1,37 @@
 import difflib
 import os
-from typing import List
+import time
+from typing import List, Tuple
 
 from prompt_toolkit.shortcuts import clear
 from prompt_toolkit.styles import merge_styles
 from robot.api import logger
 from robot.errors import ExecutionFailed, HandlerExecutionFailed
-from robot.libdocpkg.model import KeywordDoc
 from robot.libraries.BuiltIn import BuiltIn
 from robot.running.signalhandler import STOP_SIGNAL_MONITOR
+from robot.variables import is_variable
 
 from .cmdcompleter import CmdCompleter
 from .globals import context
 from .prompttoolkitcmd import PromptToolkitCmd
-from .robotkeyword import find_keyword, get_keywords, get_lib_keywords, run_command
-from .robotlib import get_builtin_libs, get_libs, match_libs
-from .sourcelines import RobotNeedUpgrade, print_source_lines, print_test_case_lines
-from .steplistener import is_step_mode, set_step_mode
+from .robotkeyword import (
+    _get_assignments,
+    _import_resource_from_string,
+    find_keyword,
+    get_keywords,
+    get_lib_keywords,
+    get_test_body_from_string,
+)
+from .robotlib import (
+    get_libraries,
+    get_libs,
+    get_resources,
+    match_libs,
+)
+from .sourcelines import (
+    print_source_lines,
+    print_test_case_lines,
+)
 from .styles import (
     BASE_STYLE,
     DEBUG_PROMPT_STYLE,
@@ -32,44 +47,25 @@ from .styles import (
 HISTORY_PATH = os.environ.get("RFDEBUG_HISTORY", "~/.rfdebug_history")
 
 
-def run_robot_command(robot_instance, command):
-    """Run command in robotframewrk environment."""
-    if not command:
-        return
-
-    result = []
-    try:
-        result = run_command(robot_instance, command)
-    except HandlerExecutionFailed as exc:
-        print_error("! FAIL:", exc.message)
-    except ExecutionFailed as exc:
-        print_error("! Expression:", command if "\n" not in command else f"\n{command}")
-        print_error("! Execution error:", str(exc))
-    except Exception as exc:
-        print_error("! Expression:", command)
-        print_error("! Error:", repr(exc))
-
-    if result:
-        for head, message in result:
-            print_output(head, message)
-
-
-class DebugCmd(PromptToolkitCmd):
+class ReplCmd(PromptToolkitCmd):
     """Interactive debug shell for robotframework."""
 
     prompt_style = DEBUG_PROMPT_STYLE
 
-    def __init__(self, completekey="tab", stdin=None, stdout=None):
-        PromptToolkitCmd.__init__(self, completekey, stdin, stdout, history_path=HISTORY_PATH)
-        self.robot = BuiltIn()
+    def __init__(self, library):
+        super().__init__(library, history_path=HISTORY_PATH)
+        self.last_keyword_exec_time = 0
+        self.listener = self.library.cli_listener or self.library.ROBOT_LIBRARY_LISTENER
+
+    def do_continue(self, args):
+        """Continue execution."""
+        return self.do_exit(args)
+
+    do_c = do_continue
 
     def get_prompt_tokens(self, prompt_text):
         return get_debug_prompt_tokens(prompt_text)
 
-    def postcmd(self, stop, line):
-        """Run after a command."""
-        return stop
-
     def pre_loop_iter(self):
         """Reset robotframework before every loop iteration."""
         reset_robotframework_exception()
@@ -77,23 +73,22 @@ class DebugCmd(PromptToolkitCmd):
     def do_help(self, arg):
         """Show help message."""
         if not arg.strip():
-            print(
+            print_output(
+                "",
                 """\
 Input Robotframework keywords, or commands listed below.
 Use "libs" or "l" to see available libraries,
 use "keywords" or "k" see the list of library keywords,
-use the TAB keyboard key to autocomplete keywords.
+use CTRL+SPACE to autocomplete keywords.
 Access https://github.com/imbus/robotframework-debug for more details.\
-"""
+""",
             )
-
-        PromptToolkitCmd.do_help(self, arg)
+        super().do_help(arg)
 
     def get_completer(self):
         """Get completer instance specified for robotframework."""
         commands = [
-            (cmd_name, cmd_name, "DEBUG command: {0}".format(doc))
-            for cmd_name, doc in self.get_helps()
+            (cmd_name, cmd_name, f"DEBUG command: {doc}") for cmd_name, doc in self.get_helps()
         ]
 
         for lib in get_libs():
@@ -101,14 +96,11 @@ Access https://github.com/imbus/robotframework-debug for more details.\
                 (
                     lib.name,
                     lib.name,
-                    "Library: {0} {1}".format(
-                        lib.name, lib.version if hasattr(lib, "version") else ""
-                    ),
+                    f"Library: {lib.name} {lib.version if hasattr(lib, 'version') else ''}",
                 )
             )
 
-        keywords: List[KeywordDoc] = get_keywords()
-        for keyword in keywords:
+        for keyword in get_keywords():
             name = f"{keyword.parent.name}.{keyword.name}"
             commands.append(
                 (
@@ -118,7 +110,11 @@ Access https://github.com/imbus/robotframework-debug for more details.\
                 )
             )
             commands.append(
-                (keyword.name, keyword.name, f"{keyword.shortdoc} [{keyword.parent.name}]")
+                (
+                    keyword.name,
+                    keyword.name,
+                    f"{keyword.shortdoc} [{keyword.parent.name}]",
+                )
             )
 
         return CmdCompleter(commands, self)
@@ -127,36 +123,52 @@ Access https://github.com/imbus/robotframework-debug for more details.\
         """Run RobotFramework keywords."""
         command = line.strip()
 
-        run_robot_command(self.robot, command)
+        self.run_robot_command(command)
+
+    def run_robot_command(self, command):
+        """Run command in robotframewrk environment."""
+        if not command:
+            return
+        result = []
+        try:
+            result = run_command(self, command)
+        except HandlerExecutionFailed as exc:
+            print_error("! FAIL:", exc.message)
+        except ExecutionFailed as exc:
+            print_error("! Expression:", command if "\n" not in command else f"\n{command}")
+            print_error("! Execution error:", str(exc))
+        except Exception as exc:
+            print_error("! Expression:", command)
+            print_error("! Error:", repr(exc))
+        if result:
+            for head, message in result:
+                print_output(head, message)
+
+    def get_rprompt_text(self):
+        """Get text for bottom toolbar."""
+        if self.last_keyword_exec_time == 0:
+            return None
+        return [("class:pygments.comment", f"# ΔT: {self.last_keyword_exec_time:.3f}s")]
 
     def _print_lib_info(self, lib, with_source_path=False):
-        print_output("   {}".format(lib.name), lib.version if hasattr(lib, "version") else "")
+        print_output(f"   {lib.name}", lib.version if hasattr(lib, "version") else "")
         if lib.doc:
-            logger.console("       {}".format(lib.doc.split("\n")[0]))
+            doc = lib.doc.split("\n")[0]
+            logger.console(f"       {doc}")
         if with_source_path:
-            logger.console("       {}".format(lib.source))
+            logger.console(f"       {lib.source}")
 
     def do_libs(self, args):
-        """Print imported and builtin libraries, with source if `-s` specified.
-
-        ls( libs ) [-s]
-        """
+        """Print imported and builtin libraries, with source if `-s` specified."""
         print_output("<", "Imported libraries:")
-        for lib in get_libs():
+        for lib in get_libraries():
             self._print_lib_info(lib, with_source_path="-s" in args)
-        print_output("<", "Builtin libraries:")
-        for name in sorted(get_builtin_libs()):
-            print_output("   " + name, "")
 
-    do_ls = do_libs
-
-    def complete_libs(self, text, line, begin_idx, end_idx):
-        """Complete libs command."""
-        if len(line.split()) == 1 and line.endswith(" "):
-            return ["-s"]
-        return []
-
-    complete_l = complete_libs
+    def do_res(self, args):
+        """Print imported and builtin libraries, with source if `-s` specified."""
+        print_output("<", "Imported resources:")
+        for lib in get_resources():
+            self._print_lib_info(lib, with_source_path="-s" in args)
 
     def do_keywords(self, args):
         """Print keywords of libraries, all or starts with <lib_name>.
@@ -172,21 +184,10 @@ Access https://github.com/imbus/robotframework-debug for more details.\
             if lib:
                 print_output("< Keywords of library", lib.name)
                 for keyword in get_lib_keywords(lib):
-                    print_output("   {}\t".format(keyword.name), keyword.shortdoc)
+                    print_output(f"   {keyword.name}\t", keyword.shortdoc)
 
     do_k = do_keywords
 
-    def complete_keywords(self, text, line, begin_idx, end_idx):
-        """Complete keywords command."""
-        if len(line.split()) == 2:
-            command, lib_name = line.split()
-            return [lib.name for lib in match_libs(lib_name)]
-        elif len(line.split()) == 1 and line.endswith(" "):
-            return [_.name for _ in get_libs()]
-        return []
-
-    complete_k = complete_keywords
-
     def do_docs(self, keyword_name):
         """Get keyword documentation for individual keywords.
 
@@ -200,7 +201,8 @@ Access https://github.com/imbus/robotframework-debug for more details.\
             logger.console(keywords[0].doc)
         else:
             print_error(
-                "< found {} keywords".format(len(keywords)), ", ".join([k.name for k in keywords])
+                f"< found {len(keywords)} keywords",
+                ", ".join([k.name for k in keywords]),
             )
 
     do_d = do_docs
@@ -208,7 +210,7 @@ Access https://github.com/imbus/robotframework-debug for more details.\
     def emptyline(self):
         """Repeat last nonempty command if in step mode."""
         self.repeat_last_nonempty_command = is_step_mode()
-        return super(DebugCmd, self).emptyline()
+        return super().emptyline()
 
     def append_command(self, command):
         """Append a command to queue."""
@@ -218,66 +220,16 @@ Access https://github.com/imbus/robotframework-debug for more details.\
         """Append exit command to queue."""
         self.append_command("exit")
 
-    def do_step(self, args):
-        """Execute the current line, stop at the first possible occasion."""
-        set_step_mode(on=True)
-        self.append_exit()  # pass control back to robot runner
-
-    do_s = do_step
-
-    def do_next(self, args):
-        """Continue execution until the next line is reached or it returns."""
-        self.do_step(args)
-
-    do_n = do_next
-
-    def do_continue(self, args):
-        """Continue execution."""
-        self.do_exit(args)
-
-    do_c = do_continue
-
-    def do_list(self, args):
-        """List source code for the current file."""
-
-        self.list_source(longlist=False)
-
-    do_l = do_list
-
-    def do_longlist(self, args):
-        """List the whole source code for the current test case."""
-
-        self.list_source(longlist=True)
-
-    do_ll = do_longlist
-
-    def list_source(self, longlist=False):
-        """List source code."""
-        if not is_step_mode():
-            print("Please run `step` or `next` command first.")
-            return
-
-        if longlist:
-            print_function = print_test_case_lines
-        else:
-            print_function = print_source_lines
-
-        try:
-            print_function(context.current_source_path, context.current_source_lineno)
-        except RobotNeedUpgrade:
-            print("Please upgrade robotframework to support list source code:")
-            print('    pip install "robotframework>=3.2" -U')
-
     def do_exit(self, args):
         """Exit debug shell."""
         set_step_mode(on=False)  # explicitly exit REPL will disable step mode
         self.append_exit()
-        return super(DebugCmd, self).do_exit(args)
+        return super().do_exit(args)
 
     def onecmd(self, line):
         # restore last command acrossing different Cmd instances
         self.lastcmd = context.last_command
-        stop = super(DebugCmd, self).onecmd(line)
+        stop = super().onecmd(line)
         context.last_command = self.lastcmd
         return stop
 
@@ -301,9 +253,83 @@ Access https://github.com/imbus/robotframework-debug for more details.\
     do_cls = do_clear
 
 
+class DebugCmd(ReplCmd):
+
+    def do_list(self, args):
+        """List source code for the current file."""
+
+        self.list_source(longlist=False)
+
+    do_l = do_list
+
+    def do_longlist(self, args=None):
+        """List the whole source code for the current test case."""
+
+        self.list_source(longlist=True)
+
+    do_ll = do_longlist
+
+    def list_source(self, longlist=False):
+        """List source code."""
+        # if not is_step_mode():
+        #     print_output("i:", "Please run `step` or `next` command first.")
+        #     return
+
+        print_function = print_test_case_lines if longlist else print_source_lines
+        print_function(
+            self.prompt_style,
+            self.library.current_source_path,
+            self.library.current_source_line,
+        )
+
+
 def reset_robotframework_exception():
     """Resume RF after press ctrl+c during keyword running."""
     if STOP_SIGNAL_MONITOR._signal_count:
         STOP_SIGNAL_MONITOR._signal_count = 0
         STOP_SIGNAL_MONITOR._running_keyword = True
         logger.info("Reset last exception of DebugLibrary")
+
+
+def set_step_mode(on=True):
+    context.in_step_mode = on
+
+
+def is_step_mode():
+    return context.in_step_mode
+
+
+def run_command(dbg_cmd, command: str) -> List[Tuple[str, str]]:
+    """Run a command in robotframewrk environment."""
+    dbg_cmd.last_keyword_exec_time = 0
+    if not command:
+        return []
+    if is_variable(command):
+        return [("#", f"{command} = {BuiltIn().get_variable_value(command)!r}")]
+    ctx = BuiltIn()._get_context()
+    if command.startswith("***"):
+        _import_resource_from_string(command)
+        return [("i:", "Resource imported.")]
+    test = get_test_body_from_string(command)
+    if len(test.body) > 1:
+        start = time.monotonic()
+        for kw in test.body:
+            kw.run(ctx)
+        dbg_cmd.last_keyword_exec_time = time.monotonic() - start
+        return_val = None
+    else:
+        kw = test.body[0]
+        start = time.monotonic()
+        return_val = kw.run(ctx)
+        dbg_cmd.last_keyword_exec_time = time.monotonic() - start
+    assign = set(_get_assignments(test))
+    if not assign and return_val is not None:
+        return [("<", repr(return_val))]
+    if assign:
+        output = []  # [("<", repr(return_val))] if return_val is not None else []
+        for variable in assign:
+            pure_var = variable.rstrip("=").strip()
+            val = BuiltIn().get_variable_value(pure_var)
+            output.append(("#", f"{pure_var} = {val!r}"))
+        return output
+    return []
